@@ -43,9 +43,11 @@ struct asm6809_options asm6809_options;
 #define OUTPUT_MOTOROLA_SREC (3)
 #define OUTPUT_INTEL_HEX (4)
 
+static int max_passes = 12;
 static int output_format = OUTPUT_BINARY;
 static char *exec_option = NULL;
 static char *output_filename = NULL;
+static char *exports_filename = NULL;
 static char *symbol_filename = NULL;
 static char *listing_filename = NULL;
 static int isa = asm6809_isa_6809;
@@ -64,8 +66,10 @@ static struct option long_options[] = {
 	{ "6309", no_argument, &isa, asm6809_isa_6309 },
 	{ "define", required_argument, NULL, 'd' },
 	{ "setdp", required_argument, &setdp, 0 },
+	{ "max-passes", required_argument, NULL, 'P' },
 	{ "output", required_argument, NULL, 'o' },
 	{ "listing", required_argument, NULL, 'l' },
+	{ "exports", required_argument, NULL, 'E' },
 	{ "symbols", required_argument, NULL, 's' },
 	{ "quiet", no_argument, NULL, 'q' },
 	{ "verbose", no_argument, NULL, 'v' },
@@ -76,20 +80,18 @@ static struct option long_options[] = {
 
 static struct slist *files = NULL;
 
-#define MAX_PASSES (10)
-
 static struct node *simple_parse_int(const char *);
 static void define_symbol(const char *);
 static void helptext(void);
 static void versiontext(void);
-static void tidy_up_and_exit(int status);
+static _Noreturn void tidy_up_and_exit(int status);
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 
 int main(int argc, char **argv) {
 
 	int c;
-	while ((c = getopt_long(argc, argv, "BDCSHe:893d:o:l:s:qv",
+	while ((c = getopt_long(argc, argv, "BDCSHe:893d:P:o:l:E:s:qv",
 				long_options, NULL)) != -1) {
 		switch (c) {
 		case 0:
@@ -121,12 +123,26 @@ int main(int argc, char **argv) {
 		case 'd':
 			define_symbol(optarg);
 			break;
+		case 'P':
+			{
+				long v = strtol(optarg, NULL, 0);
+				if (errno != 0 || v < 2 || v > 255) {
+					error(error_type_fatal, "invalid value for max-passes");
+					error_print_list();
+					tidy_up_and_exit(EXIT_FAILURE);
+				}
+				max_passes = v;
+			}
+			break;
 		case 'o':
 			output_filename = optarg;
 			break;
 		case 'l':
 			listing_filename = optarg;
 			break;
+		case 'E':
+			exports_filename = optarg;
+			break;
 		case 's':
 			symbol_filename = optarg;
 			break;
@@ -175,7 +191,7 @@ int main(int argc, char **argv) {
 	}
 
 	/* Attempt to assemble files until consistent */
-	for (unsigned pass = 0; pass < MAX_PASSES; pass++) {
+	for (unsigned pass = 0; pass < max_passes; pass++) {
 		error_clear_all();
 		listing_free_all();
 		section_set("CODE", pass);
@@ -223,7 +239,7 @@ int main(int argc, char **argv) {
 			}
 			n = node_new_int(v);
 		}
-		symbol_force_set(".exec", n, 0, MAX_PASSES);
+		symbol_force_set(".exec", n, 0, max_passes);
 	}
 
 	// XXX At the moment listing generation must precede output, as
@@ -254,11 +270,22 @@ int main(int argc, char **argv) {
 		}
 	}
 
+	/* Generate exports file */
+	if (exports_filename) {
+		FILE *expf = fopen(exports_filename, "wb");
+		if (expf) {
+			prog_print_exports(expf);
+			fclose(expf);
+		} else {
+			error(error_type_fatal, "%s: %s", exports_filename, strerror(errno));
+		}
+	}
+
 	/* Generate symbols file */
 	if (symbol_filename) {
 		FILE *symf = fopen(symbol_filename, "wb");
 		if (symf) {
-			prog_print_exports(symf);
+			prog_print_symbols(symf);
 			fclose(symf);
 		} else {
 			error(error_type_fatal, "%s: %s", symbol_filename, strerror(errno));
@@ -337,6 +364,7 @@ static void helptext(void) {
 "\n"
 "  -o, --output=FILE    set output filename\n"
 "  -l, --listing=FILE   create listing file\n"
+"  -E, --exports=FILE   create exports table\n"
 "  -s, --symbols=FILE   create symbol table\n"
 "\n"
 "  -q, --quiet     don't warn about illegal (but working) code\n"
@@ -353,7 +381,7 @@ static void helptext(void) {
 static void versiontext(void) {
 	puts(
 "asm6809 " PACKAGE_VERSION "\n"
-"Copyright (C) 2017 Ciaran Anscomb\n"
+"Copyright (C) 2018 Ciaran Anscomb\n"
 "License: GNU GPL version 3 or later <http://www.gnu.org/licenses/gpl-3.0.html>.\n"
 "This is free software: you are free to change and redistribute it.\n"
 "There is NO WARRANTY, to the extent permitted by law."
@@ -363,7 +391,7 @@ static void versiontext(void) {
 /* Call the various free_all routines to tidy up memory, then exit.  Allows us
  * to see what's been missed with valgrind. */
 
-static void tidy_up_and_exit(int status) {
+static _Noreturn void tidy_up_and_exit(int status) {
 	if (files) {
 		slist_free(files);
 		files = NULL;
