@@ -1,7 +1,7 @@
 /*
 
 asm6809, a Motorola 6809 cross assembler
-Copyright 2013-2014 Ciaran Anscomb
+Copyright 2013-2017 Ciaran Anscomb
 
 This program is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -25,10 +25,16 @@ struct asm6809_options {
 	/* Maximum program depth (include files, macros) */
 	unsigned max_program_depth;
 
+	/* Initial SETDP */
+	int setdp;
+
 	/* Can be positive or negative.  Affects error reporting. */
 	int verbosity;
+
+	/* If no listing file is required, don't keep a copy in memory. */
+	_Bool listing_required;
 };
 
 extern struct asm6809_options asm6809_options;
 
-#endif  /* ASM6809_ASM6809_H_ */
+#endif
