@@ -49,6 +49,13 @@ enum cond_state {
 	cond_state_else
 };
 
+enum translate {
+	translate_none,
+	translate_basic_keyword,
+	translate_vdg,
+	translate_inverse_vdg
+};
+
 static void set_label(struct node *label, struct node *value, _Bool changeable);
 static void args_float_to_int(struct node *args);
 static int verify_num_args(struct node *args, int min, int max, const char *op);
@@ -67,13 +74,18 @@ static void pseudo_org(struct prog_line *);
 static void pseudo_section(struct prog_line *);
 static void pseudo_section_name(struct prog_line *line);
 
+static void pseudo_fcb(struct prog_line *);
 static void pseudo_fcc(struct prog_line *);
 static void pseudo_fcn(struct prog_line *);
+static void pseudo_fcv(struct prog_line *);
+static void pseudo_fci(struct prog_line *);
 static void pseudo_fcs(struct prog_line *);
 static void pseudo_fdb(struct prog_line *);
+static void pseudo_fqb(struct prog_line *);
 static void pseudo_rzb(struct prog_line *);
 static void pseudo_fill(struct prog_line *);
 static void pseudo_rmb(struct prog_line *);
+static void pseudo_align(struct prog_line *line);
 
 static void pseudo_put(struct prog_line *);
 static void pseudo_setdp(struct prog_line *);
@@ -104,17 +116,22 @@ static struct pseudo_op pseudo_label_ops[] = {
 /* Pseudo-ops that emit data */
 
 static struct pseudo_op pseudo_data_ops[] = {
+	{ .name = "fcb", .handler = &pseudo_fcb },
 	{ .name = "fcc", .handler = &pseudo_fcc },
-	{ .name = "fcb", .handler = &pseudo_fcc },  // treat the same
 	{ .name = "fcn", .handler = &pseudo_fcn },
+	{ .name = "fcv", .handler = &pseudo_fcv },
+	{ .name = "fci", .handler = &pseudo_fci },
 	{ .name = "fcs", .handler = &pseudo_fcs },
 	{ .name = "fdb", .handler = &pseudo_fdb },
+	{ .name = "fqb", .handler = &pseudo_fqb },
 	{ .name = "rzb", .handler = &pseudo_rzb },
 	{ .name = "fzb", .handler = &pseudo_rzb },
 	{ .name = "zmb", .handler = &pseudo_rzb },  // alias
 	{ .name = "bsz", .handler = &pseudo_rzb },  // alias
 	{ .name = "fill", .handler = &pseudo_fill },
 	{ .name = "rmb", .handler = &pseudo_rmb },
+	{ .name = "align", .handler = &pseudo_align },
+	{ .name = "includebin", .handler = &pseudo_includebin },
 };
 
 /* Other pseudo-ops */
@@ -123,7 +140,6 @@ static struct pseudo_op pseudo_ops[] = {
 	{ .name = "put", .handler = &pseudo_put },
 	{ .name = "setdp", .handler = &pseudo_setdp },
 	{ .name = "include", .handler = &pseudo_include },
-	{ .name = "includebin", .handler = &pseudo_includebin },
 	{ .name = "end", .handler = &pseudo_end },
 	{ .name = "page", .handler = &pseudo_nop },
 	{ .name = "opt", .handler = &pseudo_nop },
@@ -475,7 +491,7 @@ void assemble_prog(struct prog *prog, unsigned pass) {
 		/* Other pseudo-ops */
 		op_handler = dict_lookup(pseudo_dict, n_line.opcode->data.as_string);
 		if (op_handler) {
-			listing_add_line(-1, 0, NULL, l->text);
+			listing_add_line(cur_section->pc, 0, NULL, l->text);
 			op_handler(&n_line);
 			goto next_line;
 		}
@@ -636,7 +652,8 @@ static void pseudo_section(struct prog_line *line) {
 	}
 	section_set(n->data.as_string, asm_pass);
 	node_free(n);
-	listing_add_line(-1, 0, NULL, line->text);
+	set_label(line->label, node_new_int(cur_section->pc), 0);
+	listing_add_line(cur_section->pc, 0, NULL, line->text);
 }
 
 /*
@@ -645,6 +662,8 @@ static void pseudo_section(struct prog_line *line) {
 
 static void pseudo_section_name(struct prog_line *line) {
 	section_set(line->opcode->data.as_string, asm_pass);
+	set_label(line->label, node_new_int(cur_section->pc), 0);
+	listing_add_line(cur_section->pc, 0, NULL, line->text);
 }
 
 /* PUT.  Following instructions will be located at this address.  Allows
@@ -692,8 +711,26 @@ static void pseudo_export(struct prog_line *line) {
 	}
 }
 
-static void emit_formatted(char const *ins, int argn, struct node *n, _Bool basic_token) {
-	uint8_t or_last = basic_token ? 0x80 : 0;
+static uint8_t ascii_to_vdg(uint8_t av, _Bool invert) {
+	uint8_t eor = invert ? 0x40 : 0;
+	if (av < 0x20)
+		return av ^ eor;
+	if (av < 0x40)
+		return (av+64) ^ eor;
+	if (av < 0x60)
+		return av ^ eor;
+	if (av == 0x60)
+		return 0x20 ^ eor;
+	if (av < 0x80)
+		return (av-96) ^ eor;
+	return av;
+}
+
+static void emit_formatted(char const *ins, int argn, struct node *n,
+			   enum translate mode) {
+	uint8_t or_last = (mode == translate_basic_keyword) ? 0x80 : 0;
+	_Bool invert = (mode == translate_inverse_vdg);
+	_Bool to_vdg = invert || (mode == translate_vdg);
 	switch (node_type_of(n)) {
 	default:
 		error(error_type_syntax,
@@ -717,15 +754,28 @@ static void emit_formatted(char const *ins, int argn, struct node *n, _Bool basi
 			uint8_t cc = *c;
 			if (!*(c+1))
 				cc |= or_last;
+			if (to_vdg)
+				cc = ascii_to_vdg(cc, invert);
 			section_emit_uint8(cc);
 		}
 		break;
 	}
 }
 
-/* FCC, FCB.  Embed string and byte constants. */
+/* FCB, FCC.  Embed string and byte constants. */
+
+static void pseudo_fcb(struct prog_line *line) {
+	int nargs = verify_num_args(line->args, 1, -1, "FCB");
+	if (nargs < 0)
+		return;
+	struct node **arga = node_array_of(line->args);
+	for (int i = 0; i < nargs; i++) {
+		emit_formatted("FCB", i, arga[i], 0);
+	}
+}
 
 static void pseudo_fcc(struct prog_line *line) {
+	// exactly the same as FCB, just different reporting
 	int nargs = verify_num_args(line->args, 1, -1, "FCC");
 	if (nargs < 0)
 		return;
@@ -735,6 +785,30 @@ static void pseudo_fcc(struct prog_line *line) {
 	}
 }
 
+/* FCV.  As FCC, but translates characters to VDG equivalents. */
+
+static void pseudo_fcv(struct prog_line *line) {
+	int nargs = verify_num_args(line->args, 1, -1, "FCV");
+	if (nargs < 0)
+		return;
+	struct node **arga = node_array_of(line->args);
+	for (int i = 0; i < nargs; i++) {
+		emit_formatted("FCV", i, arga[i], translate_vdg);
+	}
+}
+
+/* FCI.  As FCV, but inverse video. */
+
+static void pseudo_fci(struct prog_line *line) {
+	int nargs = verify_num_args(line->args, 1, -1, "FCI");
+	if (nargs < 0)
+		return;
+	struct node **arga = node_array_of(line->args);
+	for (int i = 0; i < nargs; i++) {
+		emit_formatted("FCI", i, arga[i], translate_inverse_vdg);
+	}
+}
+
 /* FCN.  As FCC, but zero-terminate. */
 
 static void pseudo_fcn(struct prog_line *line) {
@@ -743,7 +817,7 @@ static void pseudo_fcn(struct prog_line *line) {
 		return;
 	struct node **arga = node_array_of(line->args);
 	for (int i = 0; i < nargs; i++) {
-		emit_formatted("FCN", i, arga[i], 0);
+		emit_formatted("FCN", i, arga[i], translate_none);
 	}
 	section_emit_uint8(0);
 }
@@ -756,7 +830,7 @@ static void pseudo_fcs(struct prog_line *line) {
 		return;
 	struct node **arga = node_array_of(line->args);
 	for (int i = 0; i < nargs; i++) {
-		emit_formatted("FCS", i, arga[i], 1);
+		emit_formatted("FCS", i, arga[i], translate_basic_keyword);
 	}
 }
 
@@ -772,6 +846,18 @@ static void pseudo_fdb(struct prog_line *line) {
 	}
 }
 
+/* FQB.  Embed 32-bit constants. */
+
+static void pseudo_fqb(struct prog_line *line) {
+	int nargs = verify_num_args(line->args, 1, -1, "FQB");
+	if (nargs < 0)
+		return;
+	for (int i = 0; i < nargs; i++) {
+		long word = have_int_optional(line->args, i, "FQB", 0);
+		section_emit_uint32(word);
+	}
+}
+
 /* RZB.  Reserve zero bytes.  Additional argument specifies a non-zero fill
  * value. */
 
@@ -814,6 +900,27 @@ static void pseudo_rmb(struct prog_line *line) {
 	section_skip(count);
 }
 
+/* ALIGN.  Pad to aligned address. */
+
+static void pseudo_align(struct prog_line *line) {
+	int nargs = verify_num_args(line->args, 1, 2, "ALIGN");
+	if (nargs < 0)
+		return;
+	long align = have_int_required(line->args, 0, "ALIGN", 0);
+	long fill = have_int_optional(line->args, 1, "ALIGN", 0);
+	if (align < 1) {
+		error(error_type_out_of_range, "invalid argument to ALIGN");
+		return;
+	}
+	long count = (align - (cur_section->pc % align)) % align;
+	if (nargs < 2) {
+		section_skip(count);
+	} else {
+		for (long i = 0; i < count; i++)
+			section_emit_uint8(fill);
+	}
+}
+
 /* INCLUDE.  Nested inclusion of source files. */
 
 /* TODO: extra arguments should become available as positional variables. */
