@@ -1,7 +1,7 @@
 /*
 
 asm6809, a Motorola 6809 cross assembler
-Copyright 2013-2014 Ciaran Anscomb
+Copyright 2013-2017 Ciaran Anscomb
 
 This program is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -33,4 +33,4 @@ struct node *eval_int(struct node *n);
 struct node *eval_float_free(struct node *n);
 struct node *eval_int_free(struct node *n);
 
-#endif  /* ASM6809_EVAL_H_ */
+#endif
