@@ -1,7 +1,7 @@
 /*
 
 asm6809, a Motorola 6809 cross assembler
-Copyright 2013-2014 Ciaran Anscomb
+Copyright 2013-2017 Ciaran Anscomb
 
 This program is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -34,4 +34,4 @@ void instr_pair(struct opcode const *op, struct node const *args);
 void instr_tfm(struct opcode const *op, struct node const *args);
 void instr_reg_mem(struct opcode const *op, struct node const *args);
 
-#endif  /* ASM6809_INSTR_H_ */
+#endif
