@@ -1,7 +1,7 @@
 /*
 
 asm6809, a Motorola 6809 cross assembler
-Copyright 2013-2014 Ciaran Anscomb
+Copyright 2013-2017 Ciaran Anscomb
 
 This program is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -24,4 +24,4 @@ void interp_pop(void);
 /* Fetch positional variable from the current array. */
 struct node *interp_get(int index);
 
-#endif  /* ASM6809_INTERP_H_ */
+#endif
