@@ -1,7 +1,7 @@
 /*
 
 asm6809, a Motorola 6809 cross assembler
-Copyright 2013-2014 Ciaran Anscomb
+Copyright 2013-2017 Ciaran Anscomb
 
 This program is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -28,4 +28,4 @@ void listing_add_line(int pc, int nbytes, struct section_span const *span, char
 void listing_print(FILE *f);
 void listing_free_all(void);
 
-#endif  /* ASM6809_LISTING_H_ */
+#endif
