@@ -1,7 +1,7 @@
 /*
 
 asm6809, a Motorola 6809 cross assembler
-Copyright 2013-2014 Ciaran Anscomb
+Copyright 2013-2017 Ciaran Anscomb
 
 This program is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -18,18 +18,18 @@ option) any later version.
  */
 
 /* Output format: Binary. */
-void output_binary(const char *filename, const char *exec);
+void output_binary(const char *filename);
 
 /* Output format: DragonDOS binary. */
-void output_dragondos(const char *filename, const char *exec);
+void output_dragondos(const char *filename);
 
 /* Output format: CoCo RSDOS binary. */
-void output_coco(const char *filename, const char *exec);
+void output_coco(const char *filename);
 
 /* Output format: Motorola SREC. */
-void output_motorola_srec(const char *filename, const char *exec);
+void output_motorola_srec(const char *filename);
 
 /* Output format: Intel HEX. */
-void output_intel_hex(const char *filename, const char *exec);
+void output_intel_hex(const char *filename);
 
-#endif  /* ASM6809_OUTPUT_H_ */
+#endif
