@@ -1,7 +1,7 @@
 /*
 
 asm6809, a Motorola 6809 cross assembler
-Copyright 2013-2017 Ciaran Anscomb
+Copyright 2013-2018 Ciaran Anscomb
 
 This program is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -222,6 +222,39 @@ void prog_free_exports(void) {
 	}
 }
 
+static void print_symbol(const char *key, FILE *f) {
+	struct node *sym = symbol_get(key);
+	if (sym) {
+		struct node *n = eval_node(sym);
+		node_free(sym);
+		if (!n)
+			return;
+		int delim = 0;
+		enum node_attr old_attr = node_attr_of(n);
+		n = node_set_attr(n, node_attr_none);
+		switch (node_type_of(n)) {
+		default:
+			error(error_type_syntax, "can't export symbol type");
+			node_free(n);
+			return;
+		case node_type_int:
+		case node_type_float:
+		case node_type_reg:
+			break;
+		case node_type_string:
+			delim = '/';
+			break;
+		}
+		fprintf(f, "%s\tequ\t", key);
+		if (delim) fputc(delim, f);
+		node_print(f, n);
+		if (delim) fputc(delim, f);
+		fprintf(f, "\n");
+		n = node_set_attr(n, old_attr);
+		node_free(n);
+	}
+}
+
 static void print_exported(const char *key, void *value, FILE *f) {
 	(void)value;
 	struct prog *macro = prog_macro_by_name(key);
@@ -241,33 +274,7 @@ static void print_exported(const char *key, void *value, FILE *f) {
 		fprintf(f, "\tendm\n");
 		return;
 	}
-	struct node *sym = symbol_get(key);
-	if (sym) {
-		struct node *n = eval_node(sym);
-		node_free(sym);
-		if (!n)
-			return;
-		int delim = 0;
-		switch (node_type_of(n)) {
-		default:
-			error(error_type_syntax, "can't export symbol type");
-			node_free(n);
-			return;
-		case node_type_int:
-		case node_type_float:
-		case node_type_reg:
-			break;
-		case node_type_string:
-			delim = '/';
-			break;
-		}
-		fprintf(f, "%s\tequ\t", key);
-		if (delim) fputc(delim, f);
-		node_print(f, n);
-		if (delim) fputc(delim, f);
-		fprintf(f, "\n");
-		node_free(n);
-	}
+	print_symbol(key, f);
 }
 
 void prog_print_exports(FILE *f) {
@@ -275,3 +282,9 @@ void prog_print_exports(FILE *f) {
 		return;
 	dict_foreach(exports, (dict_iter_func)print_exported, f);
 }
+
+void prog_print_symbols(FILE *f) {
+	struct slist *symbols = symbol_get_list();
+	slist_foreach(symbols, (slist_iter_func)print_symbol, f);
+	slist_free(symbols);
+}
