@@ -1,7 +1,7 @@
 /*
 
 asm6809, a Motorola 6809 cross assembler
-Copyright 2013-2014 Ciaran Anscomb
+Copyright 2013-2017 Ciaran Anscomb
 
 This program is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -52,4 +52,4 @@ extern uint8_t const reg_stack_bit[REG_MAX];  // 0 indicates invalid, else singl
 enum reg_id reg_name_to_id(const char *name);
 const char *reg_id_to_name(enum reg_id id);
 
-#endif  /* ASM6809_REGISTERS_H_ */
+#endif
